#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from cliff import lister

from pankoclient import utils

EVENTS_LIST_COLS = ['event_type', 'generated', 'message_id', 'traits']


class CliEventList(lister.Lister):
    """List events"""

    def get_parser(self, prog_name):
        parser = super(CliEventList, self).get_parser(prog_name)
        parser.add_argument("-q", "--query", metavar="<QUERY>",
                            help="key[op]data_type::value; list. data_type "
                                 "is optional, but if supplied must be "
                                 "string, integer, float or datetime.")
        parser.add_argument("-l", "--limit", type=int, metavar="<LIMIT>",
                            help="Number of resources to return "
                                 "(Default is server default)")
        parser.add_argument("--marker", metavar="<MARKER>",
                            help="Last item of the previous listing. "
                                 "Return the next results after this value,"
                                 "the supported marker is message_id.")
        parser.add_argument("--sort", action="append",
                            metavar="<SORT_KEY:SORT_DIR>",
                            help="Sort of resource attribute, "
                                 "e.g. generated:desc")
        return parser

    def take_action(self, parsed_args):
        query = utils.cli_to_array(parsed_args.query)
        events = utils.get_client(self).events.list(
            query=query, sorts=parsed_args.sort,
            limit=parsed_args.limit, marker=parsed_args.marker)
        return utils.list2cols(EVENTS_LIST_COLS, events)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from pankoclient import utils
from pankoclient.v2 import base

from six.moves import urllib

class EventManager(base.Manager):

    url = "v2/events"

    @staticmethod
    def build_simple_query_string(q):
        """Convert list of dicts and a list of params to query url format.
        This will convert the following:
            "[{field=this,op=le,value=34},
            {field=that,op=eq,value=foo,type=string}],
            ['foo=bar','sna=fu']"
        to:
            "q.field=this&q.field=that&
            q.op=le&q.op=eq&
            q.type=&q.type=string&
            q.value=34&q.value=foo&
            foo=bar&sna=fu"
        """
        if q:
            query_params = {'q.field': [],
                            'q.value': [],
                            'q.op': [],
                            'q.type': []}

            for query in q:
                for name in ['field', 'op', 'value', 'type']:
                    query_params['q.%s' % name].append(query.get(name, ''))

            # Transform the dict to a sequence of two-element tuples in fixed
            # order, then the encoded string will be consistent in Python 2&3.
            new_qparams = sorted(query_params.items(), key=lambda x: x[0])
            q = urllib.parse.urlencode(new_qparams, doseq=True)

        return q

    def list(self, query=None, limit=None, marker=None, sorts=None):
        """List Events
        :param query: Filter arguments for which Events to return
        :type query: list
        :param limit: maximum number of resources to return
        :type limit: int
        :param marker: the last item of the previous page; we return the next
                       results after this value.
        :type marker: str
        :param sorts: list of resource attributes to order by.
        :type sorts: list of str
        """
        pagination = utils.get_pagination_options(limit, marker, sorts)
        simple_query_string = EventManager.build_simple_query_string(query)

        url = self.url
        options = []
        if pagination:
            options.append(pagination)
        if simple_query_string:
            options.append(simple_query_string)
        if options:
            url += "?" + "&".join(options)
        return self._get(url).json()



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import print_function

import logging
import os
import sys
import warnings

from cliff import app
from cliff import commandmanager
from keystoneauth1 import exceptions
from keystoneauth1 import loading

from pankoclient import client
from pankoclient import noauth
from pankoclient.v2 import capabilities_cli
from pankoclient.v2 import events_cli
from pankoclient.version import __version__


class PankoCommandManager(commandmanager.CommandManager):
    SHELL_COMMANDS = {
        "capabilities list": capabilities_cli.CliCapabilitiesList,
        "event list": events_cli.CliEventList,
    }

    def load_commands(self, namespace):
        for name, command_class in self.SHELL_COMMANDS.items():
            self.add_command(name, command_class)


class PankoShell(app.App):
    def __init__(self):
        super(PankoShell, self).__init__(
            description='Panko command line client',
            # FIXME(sileht): get version from pbr
            version=__version__,
            command_manager=PankoCommandManager(None),
            deferred_help=True,
            )

        self._client = None

    def build_option_parser(self, description, version):
        """Return an argparse option parser for this application.

        Subclasses may override this method to extend
        the parser with more global options.

        :param description: full description of the application
        :paramtype description: str
        :param version: version number for the application
        :paramtype version: str
        """
        parser = super(PankoShell, self).build_option_parser(description,
                                                            version)
        # Global arguments, one day this should go to keystoneauth1
        parser.add_argument(
            '--os-region-name',
            metavar='<auth-region-name>',
            dest='region_name',
            default=os.environ.get('OS_REGION_NAME'),
            help='Authentication region name (Env: OS_REGION_NAME)')
        parser.add_argument(
            '--os-interface',
            metavar='<interface>',
            dest='interface',
            choices=['admin', 'public', 'internal'],
            default=os.environ.get('OS_INTERFACE'),
            help='Select an interface type.'
                 ' Valid interface types: [admin, public, internal].'
                 ' (Env: OS_INTERFACE)')
        parser.add_argument(
            '--panko-api-version',
            default=os.environ.get('PANKO_API_VERSION', '2'),
            help='Defaults to env[PANKO_API_VERSION] or 2.')
        loading.register_session_argparse_arguments(parser=parser)
        plugin = loading.register_auth_argparse_arguments(
            parser=parser, argv=sys.argv, default="password")

        if not isinstance(plugin, noauth.PankoNoAuthLoader):
            parser.add_argument(
                '--panko-endpoint',
                metavar='<endpoint>',
                dest='endpoint',
                default=os.environ.get('PANKO_ENDPOINT'),
                help='Panko endpoint (Env: PANKO_ENDPOINT)')

        return parser

    @property
    def client(self):
        # NOTE(sileht): we lazy load the client to not
        # load/connect auth stuffs
        if self._client is None:
            if hasattr(self.options, "endpoint"):
                endpoint_override = self.options.endpoint
            else:
                endpoint_override = None
            auth_plugin = loading.load_auth_from_argparse_arguments(
                self.options)
            session = loading.load_session_from_argparse_arguments(
                self.options, auth=auth_plugin)

            self._client = client.Client(self.options.panko_api_version,
                                         session=session,
                                         interface=self.options.interface,
                                         region_name=self.options.region_name,
                                         endpoint_override=endpoint_override)
        return self._client

    def clean_up(self, cmd, result, err):
        if isinstance(err, exceptions.HttpError) and err.details:
            print(err.details, file=sys.stderr)

    def configure_logging(self):
        if self.options.debug:
            # --debug forces verbose_level 3
            # Set this here so cliff.app.configure_logging() can work
            self.options.verbose_level = 3

        super(PankoShell, self).configure_logging()
        root_logger = logging.getLogger('')

        # Set logging to the requested level
        if self.options.verbose_level == 0:
            # --quiet
            root_logger.setLevel(logging.ERROR)
            warnings.simplefilter("ignore")
        elif self.options.verbose_level == 1:
            # This is the default case, no --debug, --verbose or --quiet
            root_logger.setLevel(logging.WARNING)
            warnings.simplefilter("ignore")
        elif self.options.verbose_level == 2:
            # One --verbose
            root_logger.setLevel(logging.INFO)
            warnings.simplefilter("once")
        elif self.options.verbose_level >= 3:
            # Two or more --verbose
            root_logger.setLevel(logging.DEBUG)

        # Hide some useless message
        requests_log = logging.getLogger("requests")
        cliff_log = logging.getLogger('cliff')
        stevedore_log = logging.getLogger('stevedore')
        iso8601_log = logging.getLogger("iso8601")

        cliff_log.setLevel(logging.ERROR)
        stevedore_log.setLevel(logging.ERROR)
        iso8601_log.setLevel(logging.ERROR)

        if self.options.debug:
            requests_log.setLevel(logging.DEBUG)
        else:
            requests_log.setLevel(logging.ERROR)


def main(args=None):
    if args is None:
        args = sys.argv[1:]
    return PankoShell().run(args)
